/*
Looking Glass - KVM FrameRelay (KVMFR) Client
Copyright (C) 2017-2019 Geoffrey McRae <geoff@hostfission.com>
https://looking-glass.hostfission.com

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with
this program; if not, write to the Free Software Foundation, Inc., 59 Temple
Place, Suite 330, Boston, MA 02111-1307 USA
*/
#pragma once

#include "time.h"

#define INTERLOCKED_AND8        __sync_fetch_and_and
#define INTERLOCKED_OR8         __sync_fetch_and_or
#define INTERLOCKED_INC(x)      __sync_fetch_and_add((x), 1)
#define INTERLOCKED_DEC(x)      __sync_fetch_and_sub((x), 1)
#define INTERLOCKED_GET(x)      __sync_fetch_and_add((x), 0)
#define INTERLOCKED_CE(x, c, v) __sync_val_compare_and_swap((x), (c), (v))

#define INTERLOCKED_ENTER(lock) \
  while(__sync_lock_test_and_set(&(lock), 1)) while((lock));

#define INTERLOCKED_EXIT(lock) \
  __sync_lock_release(&(lock));

#define INTERLOCKED_SECTION(lock, x) \
  while(__sync_lock_test_and_set(&(lock), 1)) while((lock)); \
  x\
  __sync_lock_release(&(lock));

#define LG_LOCK_MODE    "Atomic"
typedef volatile int LG_Lock;
#define LG_LOCK_INIT(x) (x) = 0
#define LG_LOCK(x)      while(__sync_lock_test_and_set(&(x), 1)) {nsleep(100);}
#define LG_UNLOCK(x)    __sync_lock_release(&x)
#define LG_LOCK_FREE(x)
